"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var binary_heap_exports = {};
__export(binary_heap_exports, {
  ArrayBinaryMinHeap: () => ArrayBinaryMinHeap,
  arrayBinaryMaxHeap: () => arrayBinaryMaxHeap,
  arrayBinaryMinHeap: () => arrayBinaryMinHeap
});
module.exports = __toCommonJS(binary_heap_exports);
const _ArrayBinaryMinHeap = class _ArrayBinaryMinHeap {
  constructor(comparator) {
    this.comparator = comparator;
  }
  count = 0;
  items = [null];
  get length() {
    return this.count;
  }
  get unsorted() {
    return this.items.slice(1);
  }
  add(value) {
    this.items.push(value);
    this.count++;
    this.up(this.count);
  }
  addAll(values) {
    for (const value of values) {
      this.add(value);
    }
  }
  down(index) {
    let i = index;
    let current = this.items[index];
    while (i * 2 <= this.count) {
      const minChild = this.minChild(i * 2);
      const child = this.items[minChild];
      if (this.comparator(current, child) > 0) {
        this.items[i] = child;
        this.items[minChild] = current;
      }
      i = minChild;
      current = child;
    }
  }
  minChild(index) {
    if (index + 1 > this.count) {
      return index;
    }
    if (this.comparator(this.items[index], this.items[index + 1]) < 0) {
      return index;
    }
    return index + 1;
  }
  peek() {
    return this.items[1];
  }
  remove(item) {
    let removedAny;
    do {
      removedAny = false;
      for (const [value, index] of this.valuesAndIndexes()) {
        const compared = this.comparator(item, value);
        if (compared === 0) {
          this.removeAt(index);
          removedAny = true;
          break;
        }
        if (compared < 0) {
          break;
        }
      }
    } while (removedAny);
  }
  removeAt(index) {
    const value = this.items[index];
    const lastItem = this.items.pop();
    this.count--;
    if (index <= this.count) {
      this.items[index] = lastItem;
      this.down(index);
    }
    return value;
  }
  sort() {
    const values = Array(this.count + 1);
    values[0] = null;
    let index = 1;
    let anyMoves = false;
    for (const [value, originalIndex] of this.valuesAndIndexes()) {
      values[index] = value;
      if (index !== originalIndex) {
        anyMoves = true;
      }
      index++;
    }
    if (anyMoves) {
      for (let i = 1; i <= this.count; i++) {
        this.items[i] = values[i];
      }
    }
  }
  take() {
    if (this.count === 0) {
      return void 0;
    }
    return this.removeAt(1);
  }
  toArray() {
    return Array.from(this.values());
  }
  up(index) {
    let currentIndex = index;
    while (currentIndex >= 2) {
      const current = this.items[currentIndex];
      const parentIndex = currentIndex >> 1;
      const parent = this.items[parentIndex];
      const compared = this.comparator(current, parent);
      if (compared === 0) {
        this.removeAt(currentIndex);
      } else if (compared < 0) {
        this.items[parentIndex] = current;
        this.items[currentIndex] = parent;
      }
      currentIndex = parentIndex;
    }
  }
  *values() {
    for (const [value] of this.valuesAndIndexes()) {
      yield value;
    }
  }
  *valuesAndIndexes() {
    if (this.count < 1) {
      return;
    }
    const paths = [1];
    while (paths.length > 0) {
      let index = paths.shift();
      let value = this.items[index];
      const smaller = paths.map((itemIndex, pathIndex) => ({ itemIndex, otherValue: this.items[itemIndex], pathIndex })).filter(({ otherValue }) => this.comparator(otherValue, value) < 0);
      if (smaller.length > 1) {
        paths.push(index);
      } else {
        if (smaller.length === 1) {
          paths[smaller[0].pathIndex] = index;
          index = smaller[0].itemIndex;
          value = smaller[0].otherValue;
        }
        yield [value, index];
        const child1Index = index * 2;
        const child2Index = child1Index + 1;
        if (child1Index <= this.count) {
          paths.push(child1Index);
        }
        if (child2Index <= this.count) {
          paths.push(child2Index);
        }
      }
    }
  }
};
__name(_ArrayBinaryMinHeap, "ArrayBinaryMinHeap");
let ArrayBinaryMinHeap = _ArrayBinaryMinHeap;
const arrayBinaryMinHeap = /* @__PURE__ */ __name((comparator) => new ArrayBinaryMinHeap(comparator), "arrayBinaryMinHeap");
const arrayBinaryMaxHeap = /* @__PURE__ */ __name((comparator) => new ArrayBinaryMinHeap((a, b) => comparator(b, a)), "arrayBinaryMaxHeap");
//# sourceMappingURL=binary-heap.cjs.map
