var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
import { assertDefined } from "@rickosborne/guard";
const withNullsFirst = /* @__PURE__ */ __name((comparator) => {
  return /* @__PURE__ */ __name(function nullsFirst(a, b) {
    if (a == null && b == null) {
      return 0;
    } else if (a == null) {
      return -1;
    } else if (b == null) {
      return 1;
    }
    return comparator(a, b);
  }, "nullsFirst");
}, "withNullsFirst");
const withNullsLast = /* @__PURE__ */ __name((comparator) => {
  return /* @__PURE__ */ __name(function nullsLast(a, b) {
    if (a == null && b == null) {
      return 0;
    } else if (a == null) {
      return 1;
    } else if (b == null) {
      return -1;
    }
    return comparator(a, b);
  }, "nullsLast");
}, "withNullsLast");
const withDesc = /* @__PURE__ */ __name((comparator) => /* @__PURE__ */ __name(function desc(a, b) {
  return comparator(b, a);
}, "desc"), "withDesc");
const chainableComparator = /* @__PURE__ */ __name((comparator) => {
  let _nullsFirst;
  let _nullsLast;
  let _desc;
  const added = ["nullsFirst", "nullsLast", "desc"];
  return new Proxy(comparator, {
    get(target, p) {
      if (p === "nullsFirst") {
        _nullsFirst ??= chainableComparator(withNullsFirst(comparator));
        return _nullsFirst;
      }
      if (p === "nullsLast") {
        _nullsLast ??= chainableComparator(withNullsLast(comparator));
        return _nullsLast;
      }
      if (p === "desc") {
        _desc ??= chainableComparator(withDesc(comparator));
        return _desc;
      }
      return target[p];
    },
    getOwnPropertyDescriptor(target, p) {
      if (added.includes(p)) {
        return {
          enumerable: true,
          get: /* @__PURE__ */ __name(() => target[p], "get")
        };
      }
      return Object.getOwnPropertyDescriptor(target, p);
    },
    has(target, p) {
      return added.includes(p) || Object.hasOwn(target, p);
    },
    ownKeys(target) {
      return [...added, ...Object.keys(target)];
    }
  });
}, "chainableComparator");
const numberAsc = /* @__PURE__ */ __name((a, b) => a - b, "numberAsc");
const comparatorBuilder = /* @__PURE__ */ __name(() => {
  const ops = [];
  const withPrevious = /* @__PURE__ */ __name((modifier) => {
    const lastIndex = ops.length - 1;
    const comparator = ops[lastIndex][1];
    assertDefined(comparator, "Prior comparator");
    ops[lastIndex][1] = modifier(comparator);
  }, "withPrevious");
  const builder = {
    build() {
      return (a, b) => {
        for (const [accessor, comparator] of ops) {
          const aValue = accessor(a);
          const bValue = accessor(b);
          const result = comparator(aValue, bValue);
          if (result !== 0) {
            return result;
          }
        }
        return 0;
      };
    },
    get desc() {
      withPrevious((c) => withDesc(c));
      return builder;
    },
    get nullsFirst() {
      withPrevious((c) => withNullsFirst(c));
      return builder;
    },
    get nullsLast() {
      withPrevious((c) => withNullsLast(c));
      return builder;
    },
    num(accessor) {
      ops.push([accessor, (a, b) => {
        if (a == null && b == null) {
          return 0;
        }
        if (a == null) {
          return 1;
        }
        if (b == null) {
          return -1;
        }
        return a - b;
      }]);
      return builder;
    },
    str(accessor) {
      ops.push([accessor, (a, b) => {
        if (a == null && b == null) {
          return 0;
        }
        if (a == null) {
          return 1;
        }
        if (b == null) {
          return -1;
        }
        return a.localeCompare(b);
      }]);
      return builder;
    }
  };
  return builder;
}, "comparatorBuilder");
export {
  chainableComparator,
  comparatorBuilder,
  numberAsc,
  withNullsFirst,
  withNullsLast
};
//# sourceMappingURL=comparator.mjs.map
