var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
const METRIC_PREFIXES = Object.freeze([
  "",
  "K",
  "M",
  "G",
  "T",
  "P",
  "E",
  "Z",
  "Y",
  "R",
  "Q"
]);
const magnitudeOfBigInt = /* @__PURE__ */ __name((num, base) => {
  let n = num;
  let mag = 0;
  const b = BigInt(base);
  while (n >= b) {
    mag++;
    n /= b;
  }
  return mag;
}, "magnitudeOfBigInt");
const bigIntDivisorForMetric = /* @__PURE__ */ __name((magnitude, base) => {
  let n = BigInt(1);
  const b = BigInt(base);
  for (let i = 0; i < magnitude; i++) {
    n *= b;
  }
  return n;
}, "bigIntDivisorForMetric");
const formatBytes = /* @__PURE__ */ __name((bytes, base = 1024, precision = 3) => {
  const sign = bytes < 0 ? "-" : "";
  let index;
  let round;
  if (typeof bytes === "number") {
    const abs = Math.abs(bytes);
    if (abs < base) return `${bytes}B`;
    const magnitude = Math.floor(Math.log(abs) / Math.log(base));
    index = Math.min(magnitude, METRIC_PREFIXES.length - 1);
    round = (abs / base ** index).toPrecision(precision);
  } else {
    const abs = bytes < 0 ? -bytes : bytes;
    if (abs < base) return `${bytes}B`;
    const magnitude = magnitudeOfBigInt(abs, base);
    index = Math.min(magnitude, METRIC_PREFIXES.length - 1);
    const div = bigIntDivisorForMetric(index, base);
    round = (abs / div).toString(10);
  }
  const suffix = base === 1024 ? "iB" : "B";
  const prefix = METRIC_PREFIXES[index];
  return `${sign}${round}${prefix}${suffix}`;
}, "formatBytes");
export {
  METRIC_PREFIXES,
  formatBytes
};
//# sourceMappingURL=format-bytes.mjs.map
