var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
const _SkipList = class _SkipList {
  constructor(comparator, maxLevel = 8, random = () => Math.random()) {
    this.comparator = comparator;
    this.maxLevel = maxLevel;
    this.random = random;
    this.root = this.buildNode(null, this.maxLevel);
  }
  count = 0;
  level = 0;
  root;
  get length() {
    return this.count;
  }
  add(value) {
    let { current, update } = this.buildUpdate(value);
    if (current == null || this.comparator(current.value, value) !== 0) {
      const level = this.randomLevel();
      if (level > this.level) {
        for (let i = this.level + 1; i <= level; i++) {
          update[i] = this.root;
        }
        this.level = level;
      }
      const node = this.buildNode(value, level);
      for (let i = 0; i <= level; i++) {
        const toUpdate = update[i];
        if (toUpdate != null) {
          node.forward[i] = toUpdate.forward[i];
          toUpdate.forward[i] = node;
        }
      }
      this.count++;
    }
  }
  buildNode(value, level) {
    return { forward: Array(level + 1), value };
  }
  buildUpdate(value) {
    const update = Array(this.maxLevel + 1);
    let current = this.root;
    for (let i = this.level; i >= 0; i--) {
      let nextForward = current == null ? void 0 : current.forward[i];
      while (nextForward != null && this.comparator(nextForward.value, value) < 0) {
        current = nextForward;
        nextForward = nextForward.forward[i];
      }
      update[i] = current;
    }
    current = current == null ? void 0 : current.forward[0];
    return { current, update };
  }
  peek() {
    var _a;
    return (_a = this.root.forward[0]) == null ? void 0 : _a.value;
  }
  randomLevel() {
    return Math.trunc(this.random() * this.maxLevel + 1);
  }
  remove(value) {
    let { current, update } = this.buildUpdate(value);
    if (current != null && this.comparator(current.value, value) === 0) {
      for (let i = 0; i <= this.level; i++) {
        const toUpdate = update[i];
        if ((toUpdate == null ? void 0 : toUpdate.forward[i]) !== current) {
          break;
        }
        if (toUpdate != null) {
          toUpdate.forward[i] = current.forward[i];
        }
      }
      while (this.level > 0 && this.root.forward[this.level] == null) {
        this.level--;
      }
      this.count--;
    }
  }
  take() {
    if (this.count === 0) {
      return void 0;
    }
    const headValue = this.peek();
    this.remove(headValue);
    return headValue;
  }
  toArray() {
    return Array.from(this.values());
  }
  *values() {
    for (const [value] of this.valuesAndIndexes()) {
      yield value;
    }
  }
  *valuesAndIndexes() {
    let index = 0;
    let current = this.root.forward[0];
    while (current != null) {
      yield [current.value, index];
      index++;
      current = current.forward[0];
    }
  }
};
__name(_SkipList, "SkipList");
let SkipList = _SkipList;
const skipList = /* @__PURE__ */ __name((comparator) => new SkipList(comparator), "skipList");
export {
  SkipList,
  skipList
};
//# sourceMappingURL=skip-list.mjs.map
